#version 420 core

in vec2 texCoord;

layout(binding = 0) uniform sampler2D sceneTexture;
layout(binding = 1) uniform sampler2D lightSourceTexture;
layout(binding = 2) uniform sampler2D chromaticDistortionTexture;

uniform int numGhosts = 3;
uniform float brightnessGhosts;

uniform float haloWidth = 1.5;
uniform float brightnessHalo;

uniform float rOffset = 0.01;
uniform float gOffset = 0.009;
uniform float bOffset = 0.02;


out vec4 fResult;


void main()
{
	//fResult = vec4(texture(sceneTexture, texCoord.st).rgb, 1);

	vec2 texCoordFlip = -texCoord + vec2(1.0);
	vec2 deltaTexCoord = 0.75*(texCoord-texCoordFlip)*0.25;
    // Hinweis: 0.25 := 1/(cNumSamples-1)

	//Pixelfarbe Ghost
	vec4 lensflareColor = vec4(0.0);
	vec2 texCoord_new;

	for(int i = 0; i < numGhosts; i++)
    {
		texCoord_new = fract(texCoordFlip + deltaTexCoord * float(i));
		
		//chromatic aberration
		vec4 rValue = texture2D(lightSourceTexture, (texCoord_new - rOffset));  
		vec4 gValue = texture2D(lightSourceTexture, (texCoord_new - gOffset));
		vec4 bValue = texture2D(lightSourceTexture, (texCoord_new - bOffset)); 
		lensflareColor += brightnessGhosts * vec4(rValue.r, gValue.g, bValue.b, 1.0);
		
		//lensflareColor += brightnessGhosts * texture(lightSourceTexture, texCoord_new);

		//chromatic distortion
		float point = length(vec2(0.5) - texCoord_new) / length(vec2(0.5));
		lensflareColor *= texture(chromaticDistortionTexture, vec2(point, point));
    }

	vec2 flareDir = (texCoord-texCoordFlip);
    flareDir = normalize(flareDir);

	// Pixelfarbe Halo (Lichthof, in dessen Zentrum sich die Lichtquelle befindet):
	vec2 haloDir = flareDir*haloWidth;
	texCoord_new = fract(texCoordFlip+haloDir);
	lensflareColor += brightnessHalo * texture(lightSourceTexture, texCoord_new);

	fResult = texture(sceneTexture, texCoord) + lensflareColor;
}